using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Win32;
using System.Windows.Forms;
using Utils.Misc;
using PhoenixDLL.Class;

namespace PhoenixDLL.RegistryConfiguration
{
    /// <summary>
    /// Gestionnaire des options dans la base de registre
    /// </summary>
    public class RegistryManager
    {
        #region Variables
        /// <summary>
        /// Nom de l'application Phoenix
        /// </summary>
        private PhoenixSuite vPhoenixSuite;

        /// <summary>
        /// Path de base dans la base de registre pour le rpertoire d'installation
        /// </summary>
        private static readonly string RegistryPath = @"Software\Phoenix\";

        /// <summary>
        /// Chemin complet de la base de registre
        /// </summary>
        private string FullPath = "";

        /// <summary>
        /// Clef CURRENT_USER
        /// </summary>
        private RegistryKey RKey;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_Applic">L'application de la Phoenix Suite</param>
        public RegistryManager(PhoenixSuite _Applic)
        {
            vPhoenixSuite = _Applic;
            FullPath = RegistryPath + _Applic.ToString();

            RegistryKey CUkey = Registry.CurrentUser;
            RKey = CUkey.OpenSubKey(FullPath, true);
        }

        /// <summary>
        /// Constructeur
        /// </summary>
        public RegistryManager() { vPhoenixSuite = PhoenixSuite.None; }
        #endregion

        #region Mthodes
        /// <summary>
        /// Ecrit et teste dans la base de registre les informations sur l'application de la suite Phoenix 
        /// </summary>
        /// <returns>Renvoie true si la path de l'application est le mme que dans la base de registre.
        /// Sinon renvoi false</returns>
        public bool ProcessRegistry()
        {
            if (RKey == null)
            {
                RKey = Registry.CurrentUser.CreateSubKey(FullPath, RegistryKeyPermissionCheck.Default);
                string path = ApplicationPath.CommandPath;


                RKey.SetValue("Path", path);
                string buf = path + "\\Options.srz";
                buf += "";
                RKey.SetValue("Options", path + "\\Options.srz");

                if (Options.Instance.IsFirstInstance)
                {
                    try
                    {
                        Options.Instance.Path = path + "\\GestionPatient.mdb";
                        Options.Instance.IsFirstInstance = false;
                    }
                    catch (Exception) { throw; }
                }
            }
            else
            {
                string path = (string)RKey.GetValue("Path");
                if (path != null && path != ApplicationPath.CommandPath)
                    return false;
            }
            return true;
        }

        /// <summary>
        /// Rinitialise la path de l'application
        /// </summary>
        public void ResetPath()
        {
            if (vPhoenixSuite != PhoenixSuite.None)
                RKey.SetValue("Path", ApplicationPath.CommandPath);
        }

        /// <summary>
        /// Retourne le path du fichier d'option
        /// </summary>
        /// <returns></returns>
        public static string GetOptionsPath()
        {
            RegistryKey CUkey = Registry.CurrentUser;
            RegistryKey key = CUkey.OpenSubKey(RegistryPath, true);
            string Path;

            if (key == null)
            {
                key = Registry.CurrentUser.CreateSubKey(RegistryPath, RegistryKeyPermissionCheck.Default);
                Path = ApplicationPath.CommandPath + "\\Options.srz";
                key.SetValue("Options", ApplicationPath.CommandPath + "\\Options.srz");
                return Path;
            }
            else
            {
                Path = (string)key.GetValue("Options");

                if (Path == null)
                {
                    key.SetValue("Options", ApplicationPath.CommandPath + "\\Options.srz");
                    Path = ApplicationPath.CommandPath + "\\Options.srz";
                }
            }
            return Path;
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Donne ou initialise l'application de la PhoenixSuite
        /// </summary>
        public PhoenixSuite PhoenixSuite
        {
            get { return vPhoenixSuite; }
            set
            {
                vPhoenixSuite = value;
                FullPath = RegistryPath + value.ToString();
            }
        }
        #endregion
    }
}

/// <summary>
/// Enumration des logiciels de la suite Phoenix
/// </summary>
public enum PhoenixSuite
{
    /// <summary>
    /// Agenda Phoenix
    /// </summary>
    Cala,
    /// <summary>
    /// Fiches mdicales Phoenix
    /// </summary>
    Pata,
    /// <summary>
    /// Photos Phoenix
    /// </summary>
    Poto,
    /// <summary>
    /// Payement des honoraires Phoenix
    /// </summary>
    Paga,
    /// <summary>
    /// Transfert de Base de donnes Phoenix
    /// </summary>
    Traga,
    /// <summary>
    /// Dsinstalleur Phoenix
    /// </summary>
    Dela,
    /// <summary>
    /// Prescriptions Phoenix
    /// </summary>
    Pega,
    /// <summary>
    /// Statistiques Phoenix
    /// </summary>
    Stata,
    /// <summary>
    /// Suite Phoenix
    /// </summary>
    Phoenix,
    /// <summary>
    /// Aucune application de slectionne
    /// </summary>
    None,
}
